'use strict';

var fs = require('fs');
var _inplace = require('json-in-place');
var path = require('path');
var _parser = require('yargs-parser');

const NAME = 'jest-ratchet';
const getLastError = (config) => {
    const { collectCoverage, coverageReporters } = config;
    if (!collectCoverage) {
        throw new CollectCoverageError();
    }
    if ((coverageReporters || []).indexOf('json-summary') === -1) {
        throw new JsonSummaryError();
    }
};
const tryOrReject = (reject, cb) => {
    try {
        cb();
    }
    catch (e) {
        reject(e);
    }
};
class JsonSummaryError extends Error {
    constructor() {
        super(`'json-summary' needs to be listed as a coverageReporter in order for ${NAME} to work appropriately.`);
    }
}
class CollectCoverageError extends Error {
    constructor() {
        super(`'collectCoverage' option needs to be enabled in order for ${NAME} to work appropriately.`);
    }
}
class TimeoutError extends Error {
    constructor(coverageDirectory, timeout) {
        super('Jest-Ratchet timed-out waiting for the Coverage Summary');
        this.coverageDirectory = coverageDirectory;
        this.timeout = timeout;
    }
}

const FIELDS = [
    'branches',
    'functions',
    'lines',
    'statements',
];
const inplace = _inplace;
const updateFile = (fileName, result) => {
    const jestConfigRaw = fs.readFileSync(fileName, 'utf-8');
    const jestConfig = JSON.parse(jestConfigRaw);
    const prefix = jestConfig.jest ? 'jest.' : '';
    const newFile = setCoverage(jestConfigRaw, result, prefix);
    fs.writeFileSync(fileName, newFile, 'utf-8');
};
const setCoverage = (source, result, prefix) => {
    prefix += 'coverageThreshold.';
    const newSource = inplace(source);
    for (const key of Object.keys(result)) {
        for (const field of FIELDS) {
            const value = result[key][field];
            if (value) {
                newSource.set(prefix + key + '.' + field, value);
            }
        }
    }
    return newSource.toString();
};

const parser = _parser;
const findJestConfigPath = (cwd, argv) => {
    let configLocation = 'package.json';
    const args = parser(argv.slice(2));
    if (args && args.config) {
        configLocation = args.config;
    }
    if (!path.isAbsolute(configLocation)) {
        configLocation = path.resolve(cwd, configLocation);
    }
    return configLocation;
};
const findCoveragePath = (config) => {
    if (config.coverageDirectory) {
        return config.coverageDirectory;
    }
    if (config.rootDir) {
        return path.resolve(config.rootDir, 'coverage');
    }
    return path.resolve(process.cwd(), 'coverage');
};
const findCoverageSummaryPath = (coverageDirectory) => {
    return path.resolve(coverageDirectory, 'coverage-summary.json');
};

function noop() {
}

function roundToTwo(num) {
    return +(Math.round(num + 'e+2') + 'e-2');
}

const ratchetCoverage = (threshold, summary, options, rootPath) => {
    const result = {};
    if (threshold) {
        for (const key of Object.keys(threshold)) {
            const summaryKey = key === 'global' ? 'total' : key;
            if (rootPath && summaryKey.endsWith('/')) {
                result[key] = ratchetFolderCoverage(threshold[key], summaryKey, summary, options, rootPath);
            }
            else {
                result[key] = ratchetSingleCoverage(threshold[key], summary[summaryKey], options);
            }
        }
    }
    return result;
};
const ratchetFolderCoverage = (threshold, fileOrFolderPath, summary, options, rootPath) => {
    const path = fileOrFolderPath.replace('./', `${rootPath}/`);
    const jsonKeys = Object.keys(summary).slice(1);
    const calculatedSummary = {
        branches: {
            total: 0,
            covered: 0,
            skipped: 0,
            pct: 0
        },
        functions: {
            total: 0,
            covered: 0,
            skipped: 0,
            pct: 0
        },
        lines: {
            total: 0,
            covered: 0,
            skipped: 0,
            pct: 0
        },
        statements: {
            total: 0,
            covered: 0,
            skipped: 0,
            pct: 0
        }
    };
    jsonKeys.forEach((value) => {
        var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m, _o, _p, _q, _r, _s, _t, _u, _v, _w, _x, _y, _z, _0, _1, _2, _3, _4, _5, _6, _7;
        if (value.startsWith(path)) {
            const indiReport = summary[value];
            if (indiReport) {
                calculatedSummary.lines.total = ((_a = calculatedSummary.lines.total) !== null && _a !== void 0 ? _a : 0) + ((_c = (_b = indiReport.lines) === null || _b === void 0 ? void 0 : _b.total) !== null && _c !== void 0 ? _c : 0);
                calculatedSummary.lines.covered += (_e = (_d = indiReport.lines) === null || _d === void 0 ? void 0 : _d.covered) !== null && _e !== void 0 ? _e : 0;
                calculatedSummary.lines.skipped = ((_f = calculatedSummary.lines.skipped) !== null && _f !== void 0 ? _f : 0) + ((_h = (_g = indiReport.lines) === null || _g === void 0 ? void 0 : _g.skipped) !== null && _h !== void 0 ? _h : 0);
                calculatedSummary.branches.total = ((_j = calculatedSummary.branches.total) !== null && _j !== void 0 ? _j : 0) + ((_l = (_k = indiReport.branches) === null || _k === void 0 ? void 0 : _k.total) !== null && _l !== void 0 ? _l : 0);
                calculatedSummary.branches.covered += (_o = (_m = indiReport.branches) === null || _m === void 0 ? void 0 : _m.covered) !== null && _o !== void 0 ? _o : 0;
                calculatedSummary.branches.skipped = ((_p = calculatedSummary.branches.total) !== null && _p !== void 0 ? _p : 0) + ((_r = (_q = indiReport.branches) === null || _q === void 0 ? void 0 : _q.skipped) !== null && _r !== void 0 ? _r : 0);
                calculatedSummary.statements.total = ((_s = calculatedSummary.statements.total) !== null && _s !== void 0 ? _s : 0) + ((_u = (_t = indiReport.statements) === null || _t === void 0 ? void 0 : _t.total) !== null && _u !== void 0 ? _u : 0);
                calculatedSummary.statements.covered += (_w = (_v = indiReport.statements) === null || _v === void 0 ? void 0 : _v.covered) !== null && _w !== void 0 ? _w : 0;
                calculatedSummary.statements.skipped = ((_x = calculatedSummary.statements.total) !== null && _x !== void 0 ? _x : 0) + ((_z = (_y = indiReport.statements) === null || _y === void 0 ? void 0 : _y.skipped) !== null && _z !== void 0 ? _z : 0);
                calculatedSummary.functions.total = ((_0 = calculatedSummary.functions.total) !== null && _0 !== void 0 ? _0 : 0) + ((_2 = (_1 = indiReport.functions) === null || _1 === void 0 ? void 0 : _1.total) !== null && _2 !== void 0 ? _2 : 0);
                calculatedSummary.functions.covered += (_4 = (_3 = indiReport.functions) === null || _3 === void 0 ? void 0 : _3.covered) !== null && _4 !== void 0 ? _4 : 0;
                calculatedSummary.functions.skipped = ((_5 = calculatedSummary.functions.total) !== null && _5 !== void 0 ? _5 : 0) + ((_7 = (_6 = indiReport.functions) === null || _6 === void 0 ? void 0 : _6.skipped) !== null && _7 !== void 0 ? _7 : 0);
            }
        }
    });
    Object.keys(calculatedSummary).forEach((key) => {
        const value = calculatedSummary[key];
        if (value.total) {
            value.pct = roundToTwo((value.covered / value.total) * 100);
        }
    });
    const { branches, functions, lines, statements } = threshold;
    return {
        branches: ratchetSingleNumberCoverage(branches, calculatedSummary.branches, options),
        functions: ratchetSingleNumberCoverage(functions, calculatedSummary.functions, options),
        lines: ratchetSingleNumberCoverage(lines, calculatedSummary.lines, options),
        statements: ratchetSingleNumberCoverage(statements, calculatedSummary.statements, options),
    };
};
const ratchetSingleCoverage = (threshold, summary, options) => {
    const { branches, functions, lines, statements } = threshold;
    return {
        branches: ratchetSingleNumberCoverage(branches, summary.branches, options),
        functions: ratchetSingleNumberCoverage(functions, summary.functions, options),
        lines: ratchetSingleNumberCoverage(lines, summary.lines, options),
        statements: ratchetSingleNumberCoverage(statements, summary.statements, options),
    };
};
const ratchetSingleNumberCoverage = (num, category, options) => {
    if (category && typeof num === 'number') {
        const tolerance = options.tolerance
            ? Math.round(category.pct) - options.tolerance
            : category.pct;
        if (num >= 0 && num <= tolerance) {
            return options.roundDown ? Math.floor(tolerance) : tolerance;
        }
        else if (num < 0 && num >= -category.covered) {
            return -category.covered;
        }
    }
};

class JestRatchet {
    constructor(globalConfig, options = {}) {
        this.getLastError = noop;
        this.onRunComplete = noop;
        this.runResult = Promise.resolve();
        if (!process.env.DISABLE_JEST_RATCHET) {
            this.getLastError = getLastError.bind(this, globalConfig);
            this.runResult = onRunComplete(globalConfig, options);
            this.runResult.catch(e => {
                this.getLastError = () => { throw e; };
            });
        }
    }
}
const onSummaryReportComplete = (reject, resolve, watcher, timeoutTimer, coverageSummaryPath, jestConfigPath, globalConfig, options) => () => tryOrReject(reject, () => {
    watcher.close();
    if (timeoutTimer) {
        clearTimeout(timeoutTimer);
    }
    const coverageRaw = fs.readFileSync(coverageSummaryPath, 'utf-8');
    const summary = JSON.parse(coverageRaw);
    const threshold = globalConfig.coverageThreshold;
    const ratchetResult = ratchetCoverage(threshold, summary, options, globalConfig.rootDir);
    updateFile(jestConfigPath, ratchetResult);
    resolve();
});
const onRunComplete = (globalConfig, options) => new Promise((resolve, reject) => tryOrReject(reject, () => {
    const coverageDirectory = findCoveragePath(globalConfig);
    const coverageSummaryPath = findCoverageSummaryPath(coverageDirectory);
    const jestConfigPath = findJestConfigPath(process.cwd(), process.argv);
    if (!fs.existsSync(coverageDirectory)) {
        fs.mkdirSync(coverageDirectory);
    }
    if (!fs.existsSync(coverageSummaryPath)) {
        fs.closeSync(fs.openSync(coverageSummaryPath, 'w'));
    }
    const watcher = fs.watch(coverageDirectory);
    const timeout = options.timeout;
    const timeoutTimer = timeout ? setTimeout(() => {
        watcher.close();
        reject(new TimeoutError(coverageDirectory, timeout));
    }, timeout) : undefined;
    watcher.once('change', onSummaryReportComplete(reject, resolve, watcher, timeoutTimer, coverageSummaryPath, jestConfigPath, globalConfig, options));
}));

module.exports = JestRatchet;
